from __future__ import print_function

import argparse
import sys
from publisher import git


class PublisherCLI():

    def __init__(self, args):
        parser = argparse.ArgumentParser(
            description='Touch Surgery Publisher',
            usage='tspub <command> [<args>]'
        )
        subparsers = parser.add_subparsers(
            title='Available commands',
            description=(
                'These commands allow you to create and '
                'publish TS simulations'
            ),
            help='subparsers help'
        )
        cli_commands = [c for c in dir(self) if not c.startswith('_')]

        for command in cli_commands:
            sub_parser = subparsers.add_parser(
                command,
                help=getattr(self, command).__doc__
            )
            sub_parser.set_defaults(func=getattr(self, command))

        namespace = parser.parse_args(args[0:1])
        namespace.func(args[1:])

    def setup(self, args):
        """ Setup your machine, ready to create and edit simulations in 3 steps
                1. Check if git is installed
                2. Setup SSH
                3. Cloning the repo and git pull
        """
        git.setup_users_machine()

    def procedures(self, args):
        """ List all procedures in the git repository, each being a separate branch
        """
        git.list_procedures()

    def workon(self, args):
        """ Move to the branch containing the specified procedure so that it can be worked on
        """
        parser = argparse.ArgumentParser(
            description="Move to the branch containing the specified procedure",
            usage='''tspub workon <procedure_code>'''
        )
        parser.add_argument(
            'procedure',
            help="The procedure code"
        )

        args = parser.parse_args(args)
        git.change_procedure(args.procedure)


def main():
    PublisherCLI(sys.argv[1:])



# Imports
from __future__ import print_function

import sys
import os
import subprocess

# =====================================
# Define Constants
# =====================================

# Directories
HOME_DIRECTORY = os.path.expanduser("~/")
GIT_DIRECTORY = os.path.expanduser("~/git")
REPO_DIRECTORY = os.path.expanduser("~/git/content-production")

# =====================================
# Reused functions
# =====================================


def git_installed():
    try:
        subprocess.check_output(['git', '--version'])
    except OSError:
        return False

    return True


def call_command_and_print_exception(command, message):
    try:
        return subprocess.check_output(command)
    except Exception:
        print(message)
        raise


def check_and_create_directory(path):
    try:
        if not os.path.exists(path):
            os.mkdir(path)
    except Exception:
        print("Could not find or create the directory")
        raise

# =====================================
# Command functions
# =====================================


def setup_users_machine():
    """
    User Setup for publish
        1. Check if git is installed
        2. Setup SSH
        3. Cloning the repo and git pull
    """

    """
    Part 1 - Check if git is installed
    """
    if not git_installed():
        if 'win' in sys.platform.lower():
            print(
                'Please install Github for windows '
                'from https://desktop.github.com/ '
                'before coming back and continuing '
                'and running setup again.'
            )
            sys.exit(1)

    """
    Part 2 - Setup SSH
        1. Configure .ssh/config file
        2. Change directory to <user>/git
    """
    # Change to home directory directory
    os.chdir(HOME_DIRECTORY)

    # Create .ssh directory and config file
    ssh_config_stanza = (
        'Host studio-git.touchsurgery.com\n'
        ' User ubuntu\n'
        ' IdentitiesOnly true\n'
        ' IdentityFile ~/.ssh/touchsurgery-studio.pem\n'
    )
    try:
        print("Creating .ssh and config file")
        check_and_create_directory(".ssh")

        # If the file exists, check it doesn't have the texts and if it doesn't, add it
        if os.path.exists('.ssh/config'):
            with open('.ssh/config', 'r') as config_file:
                current_config_text = config_file.read()
            with open('.ssh/config', 'a') as config_file:
                if ssh_config_stanza not in current_config_text:
                    config_file.write('\n' + '\n' + ssh_config_stanza)

        # If the file doesn't exist, make it and add text
        else:
            with open('.ssh/config', 'w+') as config_file:
                config_file.write(ssh_config_stanza)
    except OSError as e:
        print("There was a problem with the ssh config setup")
        raise e

    # Move now to <user>/git for part 2
    check_and_create_directory(GIT_DIRECTORY)
    os.chdir(GIT_DIRECTORY)

    """
    Part 3 - Cloning the repo:
        1. Install lfs
        2. Clone repo
        3. Change directory to the new repo
        4. Do a git pull
    """

    # Install lfs
    print("Install lfs")
    call_command_and_print_exception(['git', 'lfs', 'install'], "lfs install failure")

    print("Config lfs")
    call_command_and_print_exception(['git', 'config', '--global', 'lfs.url',
                                      'https://live.touchsurgery.com/api/v3/lfs'], "lfs config failure")
    call_command_and_print_exception(['git', 'config', '--global', 'lfs.activitytimeout', '60'], "lfs config failure")

    # Clone repo
    if not os.path.exists("content-production"):
        print("Cloning repo")
        call_command_and_print_exception(['git', 'lfs', 'clone', 'studio-git.touchsurgery.com:/srv/git/content-repo',
                                          'content-production'], "Clone repo failure")

    # Change directory
    print("Changing directory to new git repo")
    os.chdir(REPO_DIRECTORY)

    # Git Pull
    print("Doing git pull")
    call_command_and_print_exception(['git', 'lfs', 'pull', 'origin', 'master'], "Git pull failure")


def list_procedures():
    """ Lists all current branches of the repository
    """

    if os.path.exists(REPO_DIRECTORY):
        os.chdir(REPO_DIRECTORY)
        unwanted_branches = ['master', '*', '']
        procedure_list = filter(lambda b: b not in unwanted_branches,
                                subprocess.check_output(['git', 'branch']).split())
        print(procedure_list)
        return '\n'.join(procedure_list)

    else:
        print("You do not have the content-production repository, please run the setup command.")


def change_procedure(procedure):
    """Switches git branch to the procedure selected
    """
    try:
        os.chdir(REPO_DIRECTORY)
        subprocess.check_output(['git', 'checkout', procedure])
    except Exception:
        print("Could not find the specified procedure. Make sure you have run setup and entered the correct procedure "
              "name")
        raise



import requests


class TouchSurgery(object):

    def __init__(self):
        self.auth_token = None

    def authorized(self):
        return bool(self.auth_token)

    def login(self, email, password):
        response = requests.post(
            'https://live.touchsurgery.com/api/v3/user/login',
            data=dict(
                email=email,
                password=password)
        )
        if response.status_code != 200:
            return

        payload = response.json()
        self.auth_token = payload['access_token']
